@@ -502,7 +502,7 @@ sub check_for_forged_yahoo_received_headers {
   if ($rcvd =~ /by smtp\S+\.yahoo\.com with SMTP/) { return 0; }
   my $IP_ADDRESS = IP_ADDRESS;
   if ($rcvd =~
-      /from \[$IP_ADDRESS\] by \S+\.(?:groups|scd)\.yahoo\.com with NNFMP/) {
+      /from \[$IP_ADDRESS\] by \S+\.(?:groups|scd|dcn)\.yahoo\.com with NNFMP/) {
     return 0;
   }
 
@@ -3017,11 +3017,21 @@ sub helo_ip_mismatch {
 
 sub check_all_trusted {
   my ($self) = @_;
-  if ($self->{num_relays_untrusted} > 0) {
-    return 0;
-  } else {
-    return 1;
-  }
+  return $self->{num_relays_trusted} 
+        && !$self->{num_relays_untrusted}
+        && !$self->{num_relays_unparseable};
+}
+
+sub check_no_relays {
+  my ($self) = @_;
+  return !$self->{num_relays_trusted}
+        && !$self->{num_relays_untrusted}
+        && !$self->{num_relays_unparseable};
+}
+
+sub check_relays_unparseable {
+  my ($self) = @_;
+  return $self->{num_relays_unparseable};
 }
 
 ###########################################################################
